from security.token_manager import TokenManager
import network_constants
from db.entities.layer import Layer

from typing import List, Dict
import requests
import json

def _fetch_layers(use_case: str) -> List[Layer]:
    jwt = TokenManager.getInstance().getToken()

    url = f'https://{network_constants.BUSINESS_LOGIC_HOSTNAME}:{network_constants.BUSINESS_LOGIC_REST_PORT}/api/use-cases/{use_case}/layers'

    response = requests.get(
            url,
            verify = False, 
            proxies = { "http":None, "https":None },
            headers = {"Authorization": f"Bearer {jwt}"}
    )

    if response.status_code != 200:
        raise ConnectionError(f"Could not fetch data from the server, statuscode: {response.status_code}!")

    data = json.loads(response.text)

    layers = []

    for row in data:
        layers.append(Layer.from_business_logic_dict(row))
    
    return layers

def _fetch_nodes(use_case: str, layer: Layer) -> List[Dict]:
    jwt = TokenManager.getInstance().getToken()

    url = f'https://{network_constants.SEMANTIC_LINKING_HOSTNAME}:{network_constants.SEMANTIC_LINKING_REST_PORT}/api/use-cases/{use_case}/layers/{layer.layer_name}/nodes'

    response = requests.get(
            url,
            verify = False, 
            proxies = { "http":None, "https":None },
            headers = {"Authorization": f"Bearer {jwt}"}
    )

    if response.status_code != 200:
        raise ConnectionError(f"Could not fetch data from the server, statuscode: {response.status_code}!")

    data = json.loads(response.text)

    nodes = []

    for row in data:
        nodes.append(row)
    
    return nodes

def fetch_nodes_from_semantic_linking(use_case: str):
    layers = _fetch_layers(use_case)

    print(str(layers))

    for layer in layers:
        nodes = _fetch_nodes(use_case, layer)

        print("nodes for layer: "+str(nodes))