# global imports (dont't worry, red is normal)
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase
from db.entities.schema import Schema

import pymongo
import json
from typing import List, Dict

class SchemaRepository(MongoRepositoryBase):
    '''This is a repository for MongoDb.'''

    def __init__(self):
        super().__init__(netconst.BUSINESS_LOGIC_DB_HOSTNAME,
                         netconst.BUSINESS_LOGIC_DB_PORT,
                         'rest-gateway-db')

        self._schema_collection = 'schemas'
    
    def all(self) -> List[Schema]:
        result = super().get_entries(self._schema_collection, projection={'_id': False})
        return [Schema.from_serializable_dict(row) for row in list(result)]

    def add(self, schema: Schema):
        self.put_use_case(schema.use_case)
        super().insert_entry(self._schema_collection, schema.to_serializable_dict())

    def get_for_use_case(self, use_case: str) -> Schema:
        result = super().get_entries(self._schema_collection, projection={'_id': False}, selection={"use_case": use_case})

        result = list(result)
        if len(result) > 1:
            raise ValueError("No more than 1 Schema allowed per use-case!")

        if len(result) == 1:
            return Schema.from_serializable_dict(result[0])
        
        return None
    
    def put(self, use_case: str):
        schema = self.get_for_use_case(use_case)

        if schema == None:
            schema = Schema(use_case, mappings={})
            self.add(schema)
        
        return schema
    
    def delete_for_use_case(self, use_case: str): 
        collection = self._database[self._schema_collection]
        collection.delete_one({"use_case": use_case})

    def delete_all(self):
        collection = self._database[self._schema_collection]
        collection.delete_many({})

    def update(self, schema: Schema):
        collection = self._database[self._schema_collection]
        collection.update_one({"use_case": schema.use_case}, {"$set": schema.to_serializable_dict()})
